import json

import requests


class ServiceManager:
    def __init__(self, panorama_url, api_key, verify=True):
        self.panorama_url = panorama_url
        self.api_key = api_key
        self.verify = verify
        self.url = f"{self.panorama_url}/restapi/v10.2/Objects/Services"
        self.headers = {"Content-Type": "application/json",
                        'X-PAN-KEY': f"{self.api_key}"}

    def list_services(self):
        params = {'location': 'shared', 'device-group': 'TestGroup'}
        response = requests.get(url=self.url, headers=self.headers, params=params, verify=self.verify)
        services = json.loads(response.content.decode('utf-8'))
        return services

    def create_service(self, protocol, port, name=""):
        name = name if name != "" else f"{protocol}_{port}"
        params = {'name': name, 'location': 'shared', 'device-group': 'TestGroup'}
        payload = self._get_service_payload(name=name, protocol=protocol, port=port)
        response = requests.post(url=self.url, headers=self.headers, params=params, data=json.dumps(payload),
                                 verify=self.verify)
        service = json.loads(response.content.decode('utf-8'))
        return service

    def _get_service_payload(self, name, protocol, port):
        service_payload = {
            "entry": {
                "@name": name,
                "description": name,
                "protocol": {
                    protocol.lower(): {
                        "port": port
                    },
                },
            },
        }
        return service_payload



import os
from xml.etree import ElementTree

import requests


class APICall:

    @staticmethod
    def get_api_key(panorama_url, user, password) -> str:
        user = user
        password = password
        key_response = requests.get(url=f"{panorama_url}/api/?type=keygen&user={user}&password={password}",
                                    verify=False)
        xml_key = key_response.content.decode('utf-8')
        root = ElementTree.fromstring(xml_key)
        try:
            api_key = root.find(".//key").text
            return api_key
        except AttributeError:
            return "Invalid Credentials"

    @staticmethod
    def check_valid_key(panorama_url, api_key) -> bool:
        url = f"{panorama_url}/restapi/v10.2/Panorama/Templates"
        headers = {"Content-Type": "application/json", 'X-PAN-KEY': f"{api_key}"}
        response = requests.get(url=url, headers=headers, verify=False)
        if response.status_code == 200:
            return True
        return False



import json

import requests

from panoramaTool.panorama_api.service_manager import ServiceManager


class PostRulesManager:
    def __init__(self, panorama_url, api_key, verify=True):
        self.panorama_url = panorama_url
        self.api_key = api_key
        self.verify = verify
        self.url = f"{self.panorama_url}/restapi/v10.2/Policies/SecurityPostRules"
        self.headers = {"Content-Type": "application/json",
                        'X-PAN-KEY': f"{self.api_key}"}
        self.service_manager = ServiceManager(panorama_url=panorama_url, api_key=api_key, verify=verify)

    def list_port_rules(self):
        params = {'location': 'shared', 'device-group': 'TestGroup'}
        response = requests.get(url=self.url, headers=self.headers, params=params, verify=self.verify)
        port_rules = json.loads(response.content.decode('utf-8'))
        return port_rules

    def create_post_rule(self, action, source_address, source_zone,
                         destination_address, destination_zone, protocol,
                         destination_port, name, application):
        params = {'name': name, 'location': 'shared', 'device-group': 'TestGroup'}

        if "icmp" not in protocol.lower():
            if f"{protocol}_{destination_port}" not in self.service_manager.list_services():
                self.service_manager.create_service(protocol=protocol, port=destination_port)

            payload = self._get_security_policy_payload_for_tcp_and_udp(action=action,
                                                                        source_address=source_address,
                                                                        source_zone=source_zone,
                                                                        destination_address=destination_address,
                                                                        destination_zone=destination_zone,
                                                                        protocol=protocol,
                                                                        destination_port=destination_port,
                                                                        name=name,
                                                                        application=application)
        else:
            payload = self._get_security_policy_payload_for_icmp(action=action,
                                                                 source_address=source_address,
                                                                 source_zone=source_zone,
                                                                 destination_address=destination_address,
                                                                 destination_zone=destination_zone,
                                                                 name=name)
        response = requests.post(url=self.url, headers=self.headers, params=params, data=json.dumps(payload),
                                 verify=self.verify)
        device_group = json.loads(response.content.decode('utf-8'))
        return device_group

    def _get_security_policy_payload_for_tcp_and_udp(self, action, source_address, source_zone,
                                                     destination_address, destination_zone, protocol,
                                                     destination_port, name, application):
        security_policy_payload = {
            "entry": [
                {
                    "@location": "vsys",
                    "@name": name,
                    "@vsys": "vsys1",
                    "action": action,
                    "from": {
                        "member": [
                            source_zone
                        ]
                    },
                    "to": {
                        "member": [
                            destination_zone
                        ]
                    },
                    "source": {
                        "member": [
                            source_address
                        ]
                    },
                    "destination": {
                        "member": [
                            destination_address
                        ]
                    },
                    "service": {
                        "member": [
                            f"{protocol}_{destination_port}"
                        ]
                    },
                    "application": {
                        "member": [
                            application
                        ]
                    },
                    "tag": {
                        "member": [
                            "to be reviewed"
                        ]
                    },
                }
            ]
        }
        return security_policy_payload

    def _get_security_policy_payload_for_icmp(self, action, source_address, source_zone,
                                              destination_address, destination_zone,
                                              name):
        security_policy_payload = {
            "entry": [
                {
                    "@location": "vsys",
                    "@name": name,
                    "@vsys": "vsys1",
                    "action": action,
                    "from": {
                        "member": [
                            source_zone
                        ]
                    },
                    "to": {
                        "member": [
                            destination_zone
                        ]
                    },
                    "source": {
                        "member": [
                            source_address
                        ]
                    },
                    "destination": {
                        "member": [
                            destination_address
                        ]
                    },
                    "service": {
                        "member": [
                            "application-default"
                        ]
                    },
                    "application": {
                        "member": [
                            "ping"
                        ]
                    },
                    "tag": {
                        "member": [
                            "to be reviewed"
                        ]
                    },
                }
            ]
        }
        return security_policy_payload



import os
import re

from setuptools import setup, find_packages

HERE = os.path.abspath(os.path.dirname(__file__))
PACKAGE_INIT = os.path.abspath(os.path.join('panoramaTool', '__init__.py'))

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

with open('README.md', 'r') as f:
    long_description = f.read()


def find_version(filename):
    """Attempts to find the version number in the file names filename.
    Raises RuntimeError if not found.
    """
    version = ''
    with open(filename, 'r') as fp:
        reg = re.compile(r'__version__ = [\'"]([^\'"]*)[\'"]')
        for line in fp:
            m = reg.match(line)
            if m:
                version = m.group(1)
                break
    if not version:
        raise RuntimeError('Cannot find version information')
    return version


__version__ = find_version(PACKAGE_INIT)

setup(
    name='panoramaTool',
    description='This tool is intended to insert sercurity post rules into Panorama via API.',
    version=__version__,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Timo Riedinger',
    author_email='timo.riedinger@bechtle.com',
    packages=find_packages(),
    install_requires=requirements,
    package_data={'panoramaTool': ['templates/*.html', 'static/*.css', 'csv/']},
    license='MIT',
    license_files=['LICENSE'],
    classifiers=[
        'Programming Language :: Python :: 3.11'
    ]
)
